package com.example.catchTheLetters.service.impl;

import com.auth0.jwt.interfaces.Claim;
import com.example.catchTheLetters.constant.CommonConstant;
import com.example.catchTheLetters.entity.LoginDto;
import com.example.catchTheLetters.entity.RegisterDto;
import com.example.catchTheLetters.entity.User;
import com.example.catchTheLetters.model.vo.UserVo;
import com.example.catchTheLetters.service.AuthService;
import com.example.catchTheLetters.service.RedisService;
import com.example.catchTheLetters.utils.JwtUtil;
import com.example.catchTheLetters.utils.R;
import jakarta.annotation.Resource;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Service
public class AuthServiceImpl implements AuthService {

    @Resource
    private MongoTemplate mongoTemplate;

    @Resource
    private RedisService redisService;

    @Override
    public R<String> login(LoginDto loginDto) {
        // 根据用户名查询用户信息
        String username = loginDto.getUsername();

        // 判断用户名是否为邮箱
        Pattern patternEmail = Pattern.compile(CommonConstant.EMAIL_REGEX);
        Matcher matcherEmail = patternEmail.matcher(username);

        // 判断用户名是否为手机号
        Pattern patternPhone = Pattern.compile(CommonConstant.PHONE_REGEX);
        Matcher matcherPhone = patternPhone.matcher(username);
        User user;
        if (matcherEmail.matches()){
            user = mongoTemplate.findOne(new Query(Criteria.where(CommonConstant.EMAIL).is(username).and(CommonConstant.STATUS).ne(0)), User.class);
        }
        if (matcherPhone.matches()){
            user = mongoTemplate.findOne(new Query(Criteria.where(CommonConstant.PHONE).is(username).and(CommonConstant.STATUS).ne(0)), User.class);
        }else {
            user = mongoTemplate.findOne(new Query(Criteria.where(CommonConstant.USERNAME).is(username).and(CommonConstant.STATUS).ne(0)),User.class);
        }
        if (Objects.isNull(user)){
            return R.fail("用户不存在");
        }
        if (!user.getPassword().equals(loginDto.getPassword())){
            return R.fail("密码错误");
        }
        Map<String, String> map = new HashMap<>();
        map.put("id",user.getId().toString());
        String token = JwtUtil.getToken(map, CommonConstant.TOKEN_EXPIRE_TIME);
        redisService.hSet(token, user.getId().toString(), CommonConstant.REDIS_EXPIRE_TIME);
        return R.ok(token);
    }

    @Override
    public boolean logout(String token) {
        return redisService.del(token);
    }

    @Override
    public User verify(String token) {
        Map<String, Claim> map = JwtUtil.getPayload(token);
        String id = map.get("id").asString().replaceAll("\"", "");
        User user = mongoTemplate.findOne(new Query(Criteria.where("id").is(id).and(CommonConstant.STATUS).ne(0)),User.class);
        if (Objects.isNull(user)){
            return null;
        }
        return user;
    }

    @Override
    public R<UserVo> register(RegisterDto registerDto) {
        User regedUser = getUserByName(registerDto.getUsername());

        R<UserVo> result = new R<>();

        // 用户名重复
        if (regedUser != null) {
            R.fail("用户名重复");
        }
        User user = new User();
        user.setUsername(registerDto.getUsername());
        user.setPassword(registerDto.getPassword());
        User insert = mongoTemplate.insert(user);
        if (insert != null && insert.getId() != null){
            return R.ok(user.toVo());
        }else {
            return R.fail("注册失败,请重试");
        }
    }

    @Override
    public User getUserInfo(Integer id) {
        return mongoTemplate.findOne(new Query(Criteria.where("id").is(id).and(CommonConstant.STATUS).ne(0)),User.class);
    }

    @Override
    public User getUserByName(String username) {
        return mongoTemplate.findOne(new Query(Criteria.where("username").is(username).and(CommonConstant.STATUS).ne(0)),User.class);
    }
}
